@@ -497,11 +497,12 @@ int nl_addr_cmp(struct nl_addr *a, struct nl_addr *b)
 	if (d == 0) {
 		d = a->a_len - b->a_len;
 
-		if (a->a_len && d == 0)
+		if (a->a_len && d == 0) {
 			d = memcmp(a->a_addr, b->a_addr, a->a_len);
 
 			if (d == 0)
 				return (a->a_prefixlen - b->a_prefixlen);
+		}
 	}
 
 	return d;
