@@ -154,7 +154,8 @@ char *rtnl_tc_handle2str(uint32_t handle, char *buf, size_t len)
 int rtnl_tc_str2handle(const char *str, uint32_t *res)
 {
 	char *colon, *end;
-	uint32_t h, err;
+	uint32_t h;
+	int err;
 
 	if (!strcasecmp(str, "root")) {
 		*res = TC_H_ROOT;
@@ -166,6 +167,11 @@ int rtnl_tc_str2handle(const char *str, uint32_t *res)
 		return 0;
 	}
 
+	if (!strcasecmp(str, "ingress")) {
+		*res = TC_H_INGRESS;
+		return 0;
+	}
+
 	h = strtoul(str, &colon, 16);
 
 	/* MAJ is not a number */
@@ -305,7 +311,7 @@ static int classid_map_add(uint32_t classid, const char *name)
 int rtnl_tc_read_classid_file(void)
 {
 	static time_t last_read;
-	struct stat st = {0};
+	struct stat st;
 	char buf[256], *path;
 	FILE *fd;
 	int err;
@@ -435,15 +441,16 @@ static void __init classid_init(void)
 		nl_init_list_head(&tbl_name[i]);
 
 	if ((err = rtnl_tc_read_classid_file()) < 0)
-		fprintf(stderr, "Failed to read classid file: %s\n", nl_geterror(err));
+		NL_DBG(1, "Failed to read classid file: %s\n", nl_geterror(err));
 }
 
+static void free_map(void *map) {
+	free(((struct classid_map *)map)->name);
+	free(map);
+};
+
 static void __exit classid_exit(void)
 {
-	void free_map(void *map) {
-		free(((struct classid_map *)map)->name);
-		free(map);
-	};
 	tdestroy(id_root, free_map);
 }
 /** @} */
